#include "glfHandler.h"
#include "BaseQualityHelper.h"
#include "Error.h"

char glfHandler::translateBase[16] = {0, 1, 2, 0, 3, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0};

glfHandler::glfHandler()
   {
   entries = sections = 0;
   currentEntry = currentSection = 0;
   sectionLength = 0;
   endOfSection = false;
   }

glfHandler::~glfHandler()
   {
   if (isOpen())
      Close();
   }

bool glfHandler::Open(const String & filename)
   {
   handle = ifopen(filename, "rt");
   
   if (handle==NULL) return false;
   
   ReadHeader();

   return handle != NULL;
   }


void glfHandler::ReadHeader()
{
   String magic;
   ifread(handle, magic.LockBuffer(4+1), 4 * sizeof(char));
   magic.UnlockBuffer();

   if(magic[3] != '\3')
     error("Not glf3 format\n");

   int headerTextLength;
   String headerText;
   if (ifread(handle, &headerTextLength, sizeof(int)) ==sizeof(int))
   {
     ifread(handle, headerText.LockBuffer(headerTextLength+1), headerTextLength * sizeof(char));
     headerText.UnlockBuffer();
   } 
}

bool glfHandler::isOpen()
   {
   return handle != NULL;
   }

void glfHandler::Close()
   {
   if (isOpen())
      ifclose(handle);
   }

void glfHandler::Rewind()
   {
   if (isOpen())
      ifrewind(handle);
   }

bool glfHandler::NextSection()
   {
   //if (currentEntry != entries)
      //return false;

   int labelLength;
   currentSection++;
   currentEntry = entries = 0;
   pos = 1; // zero based
   if (ifread(handle, &labelLength, sizeof(int)) == sizeof(int))
      {
      ifread(handle, label.LockBuffer(labelLength+1), labelLength * sizeof(char));
      label.UnlockBuffer();
      
      ifread(handle, &entries, sizeof(int));
      
      endOfSection = false;
      
      return true;
      }

   return false;
   }

bool glfHandler::NextEntry()
// read the next SNP entry until END of section
{
  if(endOfSection) return false;
  
  if (ifread(handle, &flag, sizeof(flag)) != sizeof(flag))
    error("Entry %d in section %s has invalid flag refBase %d rtype %d\n\n",
          currentEntry, (const char *) label, flag.refBase, flag.rtype);
          
  flag.refBase = translateBase[flag.refBase];
  
  if (flag.rtype == GLF3_RTYPE_END) // end of section
  {
    entries = currentEntry;
    endOfSection = true;
    return false;
  }
  
  else if (flag.rtype == GLF3_RTYPE_INDEL) 
  {
    uint32_t offset;
    ifread(handle, &offset, 4);
    pos += offset;
    
    uint8_t dump[8];
    ifread(handle, dump, 8);
    
    int16_t indelLengths[2];
    ifread(handle, indelLengths, 4);
    
    indelLengths[0] = abs(indelLengths[0]);
    indelLengths[1] = abs(indelLengths[1]);
    
    String indelAL1;
    ifread(handle, indelAL1.LockBuffer(indelLengths[0]+2), indelLengths[0] * sizeof(char));
    indelAL1[indelLengths[0]] = '\0';
    indelAL1.UnlockBuffer();
        
    String indelAL2;
    ifread(handle, indelAL2.LockBuffer(indelLengths[1]+2), indelLengths[1] * sizeof(char));
    indelAL2[indelLengths[1]] = '\0';
    indelAL2.UnlockBuffer();
    
    NextEntry();
  }
  
  else if (flag.rtype == GLF3_RTYPE_SNP)
  {
    if (ifread(handle, &data, (sizeof(data)-1)) == (sizeof(data)-1))
    {
      for (int i = 0; i < 10; i++)
        likelihoods[i] = bQualityConvertor.toDouble(data.lk[i]);
        
      currentEntry ++;
      
      pos += data.offset;
    
      return true;
    }
  }
  
  else
    error("Invalid flag in session %s\n\n", (const char *) label);
    
}

glfEntry & glfEntry::operator = (glfEntry & rhs)
   {
   mapQuality = rhs.mapQuality;

   for (int i = 0; i < 10; i++)
      lk[i] = rhs.lk[i];

   minLLK = rhs.minLLK;
   depth = rhs.depth;
   offset = rhs.offset;

   return * this;
   }

