#ifndef __GLF_HANDLER_H__
#define __GLF_HANDLER_H__

#include <stdint.h>
#include "InputFile.h"
#include "StringBasics.h"

#define GLF3_RTYPE_END   0
#define GLF3_RTYPE_SNP   1
#define GLF3_RTYPE_INDEL 2

struct rtypeRef
{
   /**  "XACMGRSVTWYHKDBN"[ref_base] gives the reference base */
   uint8_t refBase:4, rtype:4;
};

struct glfEntry
   {
   /** offset from the previous site, the first base in a chromosome has offset zero */
   int32_t offset;

   /** log10 minimum likelihood * 10 and the number of mapped reads */
   uint32_t depth:24, minLLK:8;

   /** maximum mapping quality for overlapping reads */
   uint8_t mapQuality;

   /** log10 likelihood ratio * 10 for genotypes AA, AC, AG, AT, CC, CG, CT, GG, GT, TT */
   uint8_t lk[10];

   glfEntry & operator = (glfEntry & rhs);
   };

class glfHandler
   {
   public:
      IFILE      handle;
      String     label;
      rtypeRef   flag;
      glfEntry   data;
      int32_t    pos; // one based
      bool       endOfSection;

      double   likelihoods[10];

      int    entries, sections;
      int    currentEntry, currentSection;
      int    sectionLength;

      glfHandler();
      ~glfHandler();

      bool Open(const String & filename);
      bool isOpen();
      void Close();
      void Rewind();

      bool NextSection();
      bool NextEntry();
      
      void ReadHeader();

      static int GenotypeIndex(int base1, int base2)
         {
         return base1 < base2 ? (base1 - 1) * (10 - base1) / 2 + (base2 - base1) :
                                (base2 - 1) * (10 - base2) / 2 + (base1 - base2);
         }

   private:
      static char translateBase[16];
   };

#endif

